/*
 * @(#)Show.java  1.0  23 August 2001
 *
 * Copyright 2001
 * College of Computer Science
 * Northeastern University
 * Boston, MA  02115
 *
 * This software may be used for educational purposes as long as
 * this copyright notice is retained intact at the top of all files.
 *
 * Should this software be modified, the words "Modified from 
 * Original" must be included as a comment below this notice.
 *
 * All publication rights are retained.  This software or its 
 * documentation may not be published in any media either in whole
 * or in part without explicit permission.
 *
 * Contact information:
 *   Richard Rasala    rasala@ccs.neu.edu
 *   Viera Proulx      vkp@ccs.neu.edu
 *   Jeff Raab         jmr@ccs.neu.edu
 * 
 * Telephone:          617-373-2462
 *
 * This software was created with support from Northeastern 
 * University and from NSF grant DUE-9950829.
 */

/**
 * A class that records the number of tickets available for a given show,
 * the prices of the adult, student, and child tickets
 * and the sales statistics
 *
 * @author Viera K. Proulx 
 * @author Richard Rasala
 * @version 23 August 2001
 */
public class Show {

    ///////////////
    // Constants //
    ///////////////
    
    // fictitious show name if none is given: The greatest show on Earth
    private static final String SHOW_NAME = "RBBBC";
    
    // initial values to display if no prices have yet been set
    private static final int ADULT_PRICE_VALUE = 15;
    private static final int STUDENT_PRICE_VALUE = 10;
    private static final int CHILD_PRICE_VALUE = 5;

    /////////////////
 // Member data //
 /////////////////

 // variables that will record the information about the show
 protected String showName = null;       // movie title
 protected int capacity;                 // theater capacity for this show
 
 // variables that record the pricing information
 protected int adultPrice;               // price of adult ticket
 protected int studentPrice;             // price of student ticket
 protected int childPrice;               // price of child ticket
 
    // variables that will record the statistics of the sales
 protected int adultsSold;               // number of adults tickets sold
 protected int studentsSold;             // number of students tickets sold
 protected int childrenSold;             // number of children tickets sold
 
 //////////////////////
 // Member functions //
 //////////////////////
 
 ////////////////////////////////////////////////////////////////////
 // Constructors and functions that set and reset the object state //
 ////////////////////////////////////////////////////////////////////
 
 // default constructor
 Show(){
 
     // imaginary show name and capacity
  showName = SHOW_NAME;
        capacity = 100;                       // 100 tickets are available

        // fictitious prices - using the pre-defined constants
  adultPrice = ADULT_PRICE_VALUE;
  studentPrice = STUDENT_PRICE_VALUE;
  childPrice = CHILD_PRICE_VALUE;
  
  // set the total sales figures to zeros
  reset();
 }
 
 
 // full constructor
 // initialize the show name, number of seats, pricing, and total sales
 Show(String name, int seats, int adult, int student, int child){
     
     // call the set function below
  setShow(name, seats, adult, student, child);
 }


//??//??//??//??//??//?? answer to question ID-4 //??//??//??//??//??//??// 
 // set the show name, number of seats, pricing, and total sales
 public void setShow(String name, int seats, 
                     int adult, int student, int child){

        // initialize the show name and seating capacity
  showName = name;
  capacity = adjust(seats);

        // set the prices as specified by the caller of the function
  adultPrice = adjust(adult);
  studentPrice = adjust(student);
  childPrice = adjust(child);
  
  // set the total sales figures to zeros
  reset();
 }

/*
//??//??//??//??//?? original version - with no error checking //??//??//??//
 // set the show name, number of seats, pricing, and total sales
 public void setShow(String name, int seats, 
                     int adult, int student, int child){

        // initialize the show name and seating capacity
  showName = name;
  capacity = seats;

        // set the prices as specified by the caller of the function
  adultPrice = adult;
  studentPrice = student;
  childPrice = child;
  
  // set the total sales figures to zeros
  reset();
 }
*/

/*
//??//??//??//??//?? answer to question ID-1 //??//??//??//??//??//??//??//
 
 // set the show name, number of seats, pricing, and total sales
 public void setShow(String name, int seats, 
                     int adult, int student, int child){

        // initialize the show name and seating capacity
  showName = name;
  if (seats < 0)
      seats = 0;
  capacity = seats;

        // set the prices as specified by the caller of the function
        if (adult < 0)
            adult = 0;
  adultPrice = adult;
  
  if (student < 0)
      student = 0;
  studentPrice = student;
  
  if (child <0)
      child = 0;
  childPrice = child;
  
  // set the total sales figures to zeros
  reset();
 }
 
*/

//??//??//??//??//??//?? answer to question ID-4 //??//??//??//??//??//??// 
 // set negative integers to zero
 private int adjust(int number){
 
     if (number < 0)
         number = 0;
     return number;
 }
 
 // start selling tickets again with the same pricing
 public void reset(){
 
  adultsSold = 0;
  studentsSold = 0;
  childrenSold = 0;  
 }

 
 //////////////////////////////////////////
 // Accessors: the set and get functions //
 //////////////////////////////////////////
 
 public void setShowName(String name){
  showName = name;
 }
 
 public String getShowName(){
  return showName;
 }
 
 public int getAdultPrice(){
  return adultPrice;
 }
 
 public int getStudentPrice(){
  return studentPrice;
 }
 
 public int getChildPrice(){
  return childPrice;
 }
 
 public int getAdultsSold(){
  return adultsSold;
 }
 
 public int getStudentsSold(){
  return studentsSold;
 }
 
 public int getChildrenSold(){
  return childrenSold;
 }
 
 // return the number of tickets still available
 public int getAvailability(){
 
     int ticketsSold = adultsSold + studentsSold + childrenSold;
  return (capacity - ticketsSold);
 }

 ///////////////////////////////////////////////////////
 // Functions that perform actions, update statistics //
 ///////////////////////////////////////////////////////
 
 // return total price
 public int price(int adults, int students, int children){
 
     int totalPrice = adults * adultPrice 
                  + students * studentPrice 
                  + children * childPrice;
  return totalPrice;
 }
 
 // sell the desired number of tickets if seats are available
 // return total price
 public int sell(int adults, int students, int children){
 
     // total number of tickets requested
  int tickets = adults + students + children;
  
  // check for availability
  if (tickets <= getAvailability()){

   // adjust the cumulative totals of tickets sold
   adultsSold += adults;
   studentsSold += students;
   childrenSold += children;
  }
   
  // return the total price of order
  return price(adults, students, children); 
 }
 
 // return true if the desired number of tickets is available
 public boolean available(int tickets){
     return (getAvailability() - tickets >= 0);
 }

 // return true if the desired number of tickets is available
 public boolean available(int adults, int students, int children){
 
     int tickets = adults + students + children;
     return (available(tickets));
 }
}